#include <cstdlib>
#include <cmath>
#include <cstdio>
#include <random>

const double PI=4.0*atan(1.0);
const double HBARC=197.3269602;

using namespace std;

int main(){
	// you may wish to change these
  double t=0.0,m=1.0,sigma=0.1,gamma=0.5,tmax=5.0;
	int nparts=1000000;
	// or you may wish to enter some interactively
	printf("Enter gamma and tmax: ");
	scanf("%lf %lf",&gamma,&tmax);
	//
	double delp,*p,*x,oldp,ebar=0.0,x2bar=0.0,guess,xx;
	double delt=0.005/gamma;
	double T=sigma*sigma/(2.0*m*gamma*delt);
	double D=T/(m*gamma);
	double ptherm=sqrt(m*T);
	int nprint=lrint(0.1/delt);
	long long int count=0,nt=0;
	p=new double[nparts];
	x=new double[nparts];
	int ipart,seed=time(NULL);
	FILE *fptr=fopen("langevin.dat","w");
	fprintf(fptr,"   t     <e>     2<e>/T    <x^2>   <x^2>analytic  ratio\n");
	mt19937 rng(seed);
	std::normal_distribution<double> gauss_dist(0,1);
	
	printf("T=%g, D=%g, gamma=%g\n",T,D,gamma);
	for(ipart=0;ipart<nparts;ipart++){
		x[ipart]=0.0;
		p[ipart]=ptherm*gauss_dist(rng);  // start with p thermalized
	}
	count=0;
	t=0;
	do{
		count+=1;
		for(ipart=0;ipart<nparts;ipart++){
			oldp=p[ipart];
			p[ipart]+=sigma*gauss_dist(rng);
			p[ipart]-=0.5*gamma*delt*(p[ipart]+oldp);
			x[ipart]+=0.5*(p[ipart]+oldp)*delt/m;
		}
		t=count*delt;
		xx=exp(-gamma*t);
		if(count%5==0){
			for(ipart=0;ipart<nparts;ipart++){
				ebar+=0.5*p[ipart]*p[ipart]/m;
			}
			nt+=nparts;
		}
		if(count%nprint==0){
			x2bar=0.0;
			for(ipart=0;ipart<nparts;ipart++){
				x2bar+=x[ipart]*x[ipart];
			}
			guess=2.0*D*(t-(1.0-xx)/gamma);
			printf("t=%7.3f: ebar=%7.5f, ebar/(T/2)=%7.4f, x2bar=%9.6f =? %9.6f, ratio=%7.4f -> %7.4f\n",
			t,ebar/double(nt),ebar*2.0/(double(nt)*T),x2bar/double(nparts),guess,x2bar/(nparts*guess),x2bar/(2*D*t*nparts));
			fprintf(fptr,"%7.3f %7.5f %7.4f %9.6f %9.6f %7.4f\n",
			t,ebar/double(nt),ebar*2.0/(double(nt)*T),x2bar/double(nparts),guess,x2bar/(nparts*guess));
		}
	}while(t<tmax);
	fclose(fptr);
	return 0;
}


